PROGRAM TALAnemo
   !---------------------------------------------------------------------------------------
   !     Programm zur objektiven Bestimmung eines Ersatz-Anemometerstandortes auf Basis von
   !     Windfeldern einer diagnostischen oder prognostischen Windfeldbibliothek
   !     fr AUSTAL2000 bzw. LASAT.
   !     Alle im Verzeichnis der Windfeldbibliothek gefundenen Dateien werden ausgewertet,
   !     wenn sie folgender Namenskonvention entsprechen:
   !     wSXDDaNN.dmna mit S : 1-6 entspricht AK-Klasse
   !                       X : beliebig
   !                       DD: 01-36 entspricht Antriebswindrichtung in Dekagrad
   !                           sn    entspricht Antriebswindrichtung 180
   !                           we    entspricht Antriebswindrichtung 270
   !                       NN: 00    bei ungenestetem Gitter
   !                           11-N1 bei genesteten Gittern 1 bis N
   !
   !     Aufruf: TALAnemo.exe   Verzeichnis_der_Windfeldbibliothek
   !
   !     Das Programm nutzt die Routine GetArg, die kein FORTRAN 95 Standard ist. Alternativ
   !     kann ab FORTRAN 2003 die Routine get_command_argument verwendet werden.
   !---------------------------------------------------------------------------------------
   !
   USE mo_global
   !
   IMPLICIT NONE
   !
   INTEGER        :: idvalue(8) ,istat
   CHARACTER*10   :: ddate,dtime,dzone
   CHARACTER*5    :: carg2
   CHARACTER*280  :: cl
   !
   INTEGER        :: nMode ,jGrid ,jak ,jdd ,nCallsH ,nCallsD ,iRand
   REAL           :: xwest ,xost ,ysued ,ynord
   REAL ,ALLOCATABLE :: u1d(:) ,v1d(:)
   !
   nGrids    = -9
   nDD       = -9
   !
   call getarg(1,clib,istat)
   if ( istat < 0 ) then
     write (*,*)
     write (*,'(A)') 'TAL-Anemo berechnet eine objektive Ersatzanemometerposition (EAP)'
     write (*,'(A)') 'nach VDI-Richtlinie 3783 Blatt 16. Der Programmaufruf muss in einem'
     write (*,'(A)') 'Windows-Kommandofenster erfolgen und nach dem Programmnamen das'
     write (*,'(A)') 'Verzeichnis enthalten, in dem die Windfeldbibliothek vorliegt. Der optionale'
     write (*,'(A)') 'Parameter -l oder -long bewirkt die Ausgabe der Guetemae aller Gitterpunkte'
     write (*,'(A)') 'mit stetiger Winddrehung in die Datei talanemo.lgm:'
     write (*,*)
     write (*,'(A)') 'TAL-ANEMO  Verzeichnis-Windfeldbibliothek [-l|-long]'
     write (*,*)
     PAUSE 'Weiter mit RETURN-Taste ...'
     STOP
   endif
   !
   lLong = .false. ; carg2 = ' '
   call getarg(2,carg2,istat)
   if ( carg2 == '-l' .or. carg2 == '-long' .or. carg2 == '-L' .or. carg2 == '-Long' ) lLong = .true.
   !
   OPEN(99,FILE='TAL-Anemo.log',IOSTAT=istat)
   if ( istat < 0 ) then
     write (* ,'(A)') 'Fehler: Protokolldatei TAL-Anemo.log kann nicht angelegt werden.'
     write (99,'(A)') 'Fehler: Protokolldatei TAL-Anemo.log kann nicht angelegt werden.'
     PAUSE 'Weiter mit RETURN-Taste ...'
     STOP
   endif
   CALL DATE_AND_TIME(ddate,dtime,dzone,idvalue)
   WRITE (99,'(4A,I4.4,A,I2.2,A,I2.2,1x,I2.2,A,I2.2,A,I2.2)') &
         cProgV,' Build: ',cBuildTime,' gestartet um ',idvalue(1),'-',idvalue(2),'-',idvalue(3),idvalue(5),':',idvalue(6),':',idvalue(7)
   !
   call sub_Read1d()
   !
   write (99,'(3A)') 'Bibliotheksverzeichnis ist "',TRIM(cLib),'"'
   !
   write (99,*)
   write (99,'(A)')        '-----------------------------------------------------------------------------------------------'
   write (99,'(A)')        'Mindestanforderungen fuer Eignung von Modellgitterpunkten als Ersatz-Anemometerstandort:'
   write (99,'(A,I2)'    ) 'Anzahl nicht ausgewerteter Randpunkte im aeusseren Gitter: ',nRandPkte
   write (99,'(A,F4.1,A)') 'Windgeschwindigkeit immer groesser oder gleich ..........: ',ffmin,' m/s'
   write (99,'(A)')        '-----------------------------------------------------------------------------------------------'
   !
   call sub_ReadDir()
   write (99,*)
   write (99,'(A)') '==============================================================================================================='
   write (99,'(A)') '==================    Objektiv bestimmte Ersatz-Anemometerorte im Gitter 1 je Modellebene:    ================='
   write (99,'(A)') '==============================================================================================================='
   write (99,*)
   !
   nCallsH = 0
   jGrid   = 1
   !
   nKorr = 0
   do jak=1,nAKS
      nCallsD = 0
      if ( nCallsH == 0 ) then
         !
         if ( cwFiles(jak,1,jGrid) == ' ' ) then
            write (99,'(A,I1,A)') 'Fehler: Zu AK-Klasse Nr. ',jak,' existiert keine Bibliotheksdatei.'
            STOP 'Fehler: Bibliotheksdatei existiert nicht'
         endif
         nCallsH = nCallsH + 1
         call sub_ReadHeader(nCallsH,nMode,cwFiles(jak,1,jgrid))
         !
         xwest = refx + xmin + delta * iRand
         xost  = refx + xmin + delta * (hghb(1)-lowb(1)-2*iRand)
         ysued = refy + ymin + delta * iRand
         ynord = refy + ymin + delta * (hghb(2)-lowb(2)-2*iRand)
         write (99,'(A,I2,2(A,F9.0))') 'Auswertegebiet Gitter ',jGrid,'  West - Ost : ',xwest,' bis ',xost
         write (99,'(24x, 2(A,F9.0))')                                '  Sued - Nord: ',ysued,' bis ',ynord
         !
         ALLOCATE ( u      (lowb(1):hghb(1),lowb(2):hghb(2),lowb(3):hghb(3))  &
                   ,v      (lowb(1):hghb(1),lowb(2):hghb(2),lowb(3):hghb(3))  &
                   ,w      (lowb(1):hghb(1),lowb(2):hghb(2),lowb(3):hghb(3))  &
                   ,zp     (lowb(1):hghb(1),lowb(2):hghb(2),lowb(3):hghb(3))  &
                   ,zlvl   (                                lowb(3):hghb(3))  &
                   ,dd     (lowb(1):hghb(1),lowb(2):hghb(2),lowb(3):hghb(3))  &
                   ,ddm1   (lowb(1):hghb(1),lowb(2):hghb(2),lowb(3):hghb(3))  &
                   ,ff     (lowb(1):hghb(1),lowb(2):hghb(2),lowb(3):hghb(3))  &
                   ,dddelta(lowb(1):hghb(1),lowb(2):hghb(2),lowb(3):hghb(3))  &
                   ,ffact  (lowb(1):hghb(1),lowb(2):hghb(2),lowb(3):hghb(3))  &
                   ,dddact (lowb(1):hghb(1),lowb(2):hghb(2),lowb(3):hghb(3))  &
                   ,lstetig(lowb(1):hghb(1),lowb(2):hghb(2),lowb(3):hghb(3))    )
         u       = 0.
         v       = 0.
         w       = 0.
         zp      = 0.
         zlvl    = 0.
         dd      = 0.
         ddm1    = 0.
         ff      = 0.
         dddelta = 0.
         ffact   = 0.
         dddact  = 0.
         lStetig = .true.
         lStetig(0,:,:) = .false.
         lStetig(:,0,:) = .false.
         lStetig(:,:,0) = .false.
         !
         ALLOCATE ( Sum1d  (                                lowb(3):hghb(3))  &
                   ,Sum3d  (lowb(1):hghb(1),lowb(2):hghb(2),lowb(3):hghb(3))  &
                   ,Sum1d1d(                                lowb(3):hghb(3))  &
                   ,Sum3d3d(lowb(1):hghb(1),lowb(2):hghb(2),lowb(3):hghb(3))  &
                   ,Sum1d3d(lowb(1):hghb(1),lowb(2):hghb(2),lowb(3):hghb(3))  &
                   ,gdxyz_c(lowb(1):hghb(1),lowb(2):hghb(2),lowb(3):hghb(3))  &
                   ,gfxyz_c(lowb(1):hghb(1),lowb(2):hghb(2),lowb(3):hghb(3))  &
                   ,gdxy_c (lowb(1):hghb(1),lowb(2):hghb(2)                )  &
                   ,gfxy_c (lowb(1):hghb(1),lowb(2):hghb(2)                )  &
                   ,u1d(lowb(3)+1:hghb(3)) ,v1d(lowb(3)+1:hghb(3))              )
         Sum1d   = 0.
         Sum3d   = 0.
         Sum1d1d = 0.
         Sum3d3d = 0.
         Sum1d3d = 0.
         gdxyz_c = 0.
         gfxyz_c = 0.
         gdxy_c  = 0.
         gfxy_c  = 0.
         u1d     = -999.
         v1d     = -999.
      end if
      !
      do jDD=nDD,1,-1
         if ( cwFiles(jAK,jDD,jgrid) == ' ' ) CYCLE
         nCallsH = nCallsH + 1
         call sub_ReadHeader(nCallsH,nMode,cwFiles(jak,jDD,jgrid))
         nCallsD = nCallsD + 1
         call sub_ReadWind  (nCallsD,nMode,nCallsH)
         ddHm1 = ddFiles(jak,jdd,jgrid)
         !
         if ( n2Basis == 1 ) then
            ddHm1 = ddHm1 + 180.
            if ( ddHm1 >= 360. ) ddHm1 = ddHm1 - 360.
         end if
         !
         EXIT
      end do
      !
      do jdd=1,nDD
         if ( cwFiles(jAK,jDD,jgrid) == ' ' ) CYCLE
         write (*,'(2A)') '... analysiere Datei ',TRIM(cwFiles(jak,jDD,jgrid))
         nCallsH = nCallsH + 1
         call sub_ReadHeader(nCallsH,nMode,cwFiles(jak,jDD,jgrid))
         nCallsD = nCallsD + 1
         call sub_ReadWind  (nCallsD,nMode,nCallsH)
         ddH = ddFiles(jak,jdd,jgrid)
         !
         call sub_CalcQC(jak,jdd,jgrid)
      end do
   !
   end do
   !
   call sub_CalcQC2()
   !
   !
   if ( lLong ) then
     OPEN(98,FILE='TAL-Anemo.lgm',IOSTAT=istat)
     if ( istat < 0 ) then
       write (* ,'(A)') 'Fehler: Protokolldatei TAL-Anemo.lgm kann nicht angelegt werden.'
       write (99,'(A)') 'Fehler: Protokolldatei TAL-Anemo.lgm kann nicht angelegt werden.'
       PAUSE 'Weiter mit RETURN-Taste ...'
       STOP
     endif
   endif
   write (*,'(A)') '... Anemometerposition wird bestimmt'
   call sub_CalcAnemo(jgrid)
   !
   deallocate (u,v,w,zp,zlvl,dd,ddm1,ff,ffact,dddelta,dddact,lstetig)
   deallocate (Sum1d ,Sum3d ,Sum1d1d ,Sum3d3d ,Sum1d3d ,gdxyz_c ,gfxyz_c ,gdxy_c ,gfxy_c ,u1d ,v1d  )
   !
   CALL DATE_AND_TIME(ddate,dtime,dzone,idvalue)
   write (99,*)
   WRITE (99,'(2A,I4.4,A,I2.2,A,I2.2,1x,I2.2,A,I2.2,A,I2.2)') cProgV,' beendet um '  &
         ,idvalue(1),'-',idvalue(2),'-',idvalue(3),idvalue(5),':',idvalue(6),':',idvalue(7)
   CLOSE (99)
   write (*,*) 'TAL-Anemo beendet. Protokolldatei TAL-Anemo.log erstellt.'
   !
END PROGRAM TALAnemo
